
package boundedfigure;

public class Circle extends BoundedBox implements Intersector {
    
    private int xC, yC, radius ;
    
    public Circle() {
        setX(1) ;
        setY(1) ;
        setRadius(1) ;
    }
    
    public Circle(int x, int y, int radius) throws IllegalArgumentException {
        setX(x) ;
        setY(y) ;
        setRadius(radius) ;
    }
    
    public void setX(int x) {
        xC = x ;
    }
    
    public int getX() {
        return xC ;
    }
    
    public void setY(int y) {
        yC = y ;
    }
    
    public int getY() {
        return yC ;
    }
    
    public void setRadius(int radius) throws IllegalArgumentException {
        if (radius<0)
            throw new IllegalArgumentException("negative radius") ;
        this.radius = radius ;
    }
    
    public int getRadius() {
        return radius ;
    }
    
    public int getXBound() {
        return xC - radius ;
    }
    public int getYBound() {
        return yC - radius ;
    }
    public int getWidthBound() {
        return 2*radius ;
    }
    public int getHeightBound() {
        return 2*radius ;
    }
    
    public boolean intersects(Object obj) throws IllegalArgumentException {
        if (obj instanceof Circle) {
            Circle c2 = (Circle) obj ;
            int xC2 = c2.xC ;
            int yC2 = c2.yC ;
            int radius2 = c2.radius ;
            return (xC-xC2)*(xC-xC2) + (yC-yC2)*(yC-yC2)
                    < (radius+radius2)*(radius+radius) ;
        } else if (obj instanceof Rectangle) {
            Rectangle r2 = (Rectangle) obj ;
            int rXlow  = r2.getX() ;
            int rYlow  = r2.getY() ;
            int rXhigh = rXlow + r2.getWidth() ;
            int rYhigh = rYlow + r2.getHeight() ;
            //
            //  Center of circle is located in one of these nine regions
            //                rXlow           rXhigh
            //                  |               |
            //        (1)       |      (4)      |     (7)
            //                  |               |
            //      rYlow  -----------------------------
            //                  |               |
            //        (2)       |      (5)      |     (8)
            //                  |               |
            //      rYhigh -----------------------------
            //                  |               |
            //        (3)       |      (6)      |     (9)
            //                  |               |
            
            if (xC<rXlow) {
                if (yC<rYlow) {                      // (1)
                    return inside(rXlow, rYlow) ;
                } else if (yC>rXhigh) {              // (3)
                    return inside(rXlow, rYhigh) ;
                } else { // rYlow <= yC <= rYhigh    // (2)
                    return xC + radius >= rXlow ;
                }
            } else if (xC>rXhigh) {
                if (yC<rYlow) {                      // (7)
                    return inside(rXhigh, rYlow) ;
                } else if (yC>rYhigh) {
                    return inside(rXhigh, rYhigh) ;  // (9)
                } else { // rYlow <= yC <= rYhigh
                    return xC - radius <= rXhigh ;   // (8)
                }                
            } else { // rXlow <= xC <= rXhigh
                if (yC<rYlow) {
                    return inside(xC, rYlow) ;       // (4)
                } else if (yC>rYhigh) {
                    return inside(xC, rYhigh) ;      // (6)
                } else { // rYlow <= yC <= rYhigh
                    return true ;                    // (5)
                }
            }   
        } else {
            String excpMsg = "Don't know how to intersect Circle and "
                   + obj.getClass().getName() ;
            throw new IllegalArgumentException(excpMsg) ;
        }
    }
    
    public boolean inside(int x, int y) {
        return (xC-x)*(xC-x) + (yC-y)*(yC-y) < radius*radius ;
    }
    
}
